@@ -33,7 +33,7 @@ from typing import (
 )
 import zipfile
 
-from .common import _parents
+from .common import _parents, match_glob
 
 __all__ = (
     "ZipPath",
@@ -290,6 +290,22 @@ class ZipPath:
         if not found_name:
             raise FileNotFoundError(f"No such file or directory: '{self.at}'")
 
+    def glob(self, pattern: str, include_virtual: bool = True):
+        """Iterate over this subtree and yield all existing files (of any
+        kind, including directories) matching the given relative pattern.
+
+        :param pattern: the pattern to match (e.g. use ``**/*`` to yield all)
+        :param include_virtual: whether to yield paths that are not stored in the tar index
+
+        """
+        iterator = (
+            self._all_at_set()
+            if include_virtual
+            else {name.rstrip("/") for name in self._zipfile.namelist()}
+        )
+        for name in match_glob(self.at, pattern, iterator):
+            yield self.__class__(self, at=name)
+
     # shutil like interface
 
     def putfile(self, path: Union[str, Path]):
@@ -371,6 +387,47 @@ class ZipPath:
                 )
                 self._zipfile.write(subpath, zippath)
 
+    def extract_tree(
+        self,
+        outpath: Union[str, Path],
+        *,
+        pattern: str = "**/*",
+        callback: Optional[Callable[[str, Any], None]] = None,
+        cb_descript: str = "Extracting objects",
+    ):
+        """Extract the archive path (and recursive children) to an external path.
+
+        :param outpath: The path to output to
+        :param pattern: the glob pattern for selecting children to extract
+
+        :param callback: a callback to report on the process, ``callback(action, value)``,
+            with the following callback signatures:
+
+            - ``callback('init', {'total': <int>, 'description': <str>})``,
+                to signal the start of a process, its total iterations and description
+            - ``callback('update', <int>)``,
+                to signal an update to the process and the number of iterations to progress
+
+        :param cb_descript: the description to return in the callback
+
+        """
+        outpath = cast(str, os.path.abspath(outpath))
+
+        if callback is None:
+            callback = lambda action, value: None  # noqa: E731
+        else:
+            callback("init", {"total": 1, "description": "Counting objects to extract"})
+            count = sum(1 for _ in self.glob(pattern, include_virtual=False))
+            callback("init", {"total": count, "description": cb_descript})
+
+        for path in self.glob(pattern, include_virtual=False):
+            callback("update", 1)
+            try:
+                info = self._zipfile.getinfo(path.at)
+            except KeyError:
+                info = self._zipfile.getinfo(path.at + "/")
+            self._zipfile.extract(path=outpath, member=info)
+
 
 class FileList(Sequence):
     """A list of ``zipfile.ZipInfo`` which mirrors the ``zipfile.ZipFile.NameToInfo`` mapping.
